from itertools import iteritems


def inverse(mapping):
    """
    Returns a new dictionary with keys and values swapped.

        >>> inverse({1: 2, 3: 4})
        {2: 1, 4: 3}
    """
    return dict([
        (value, key)
        for (key, value) in
        iteritems(mapping)])


def find(dictionary, element):
    """
    Returns a key whose value in `dictionary` is `element`
    or, if none exists, None.

        >>> d = {1:2, 3:4}
        >>> find(d, 4)
        3
        >>> find(d, 5)
    """
    for (key, value) in iteritems(dictionary):
        if element is value:
            return key




from . import pm
from .linuxtools import lxrun
from .linuxtools import lxget
from .linuxtools import SPEC

from . import linuxtools as linux
from . import formtools as format
from . import filetools as futils



import sqlparse as sp


def update_or_delete_to_select(statement):
    return 'SELECT * FROM ' + \
           str(list(filter(
               lambda x: isinstance(x, sp.sql.Identifier),
               statement))[0])\
           + ' ' + \
           str(list(filter(
               lambda x: isinstance(x, sp.sql.Where),
               statement))[-1])


# Input a sql
# Return a select only SQL list
def selectify(sentence: str):
    for statement in sp.parse(sentence):
        if statement.get_type() in ['UPDATE', 'DELETE']:
            yield update_or_delete_to_select(statement)


a = selectify('update eee where id = 4; update aaa where id =4')
print(list(a))


def to_str(sth):
    if isinstance(sth, str):
        return sth
    elif isinstance(sth, bytes):
        return sth.decode()
    else:
        return str(sth)


def to_bytes(sth):
    if isinstance(sth, bytes):
        return sth
    elif isinstance(sth, str):
        return sth.encode()
    else:
        return bytes(sth)


def to_addr(strs):
    host = 'localhost'
    port = 0
    if isinstance(strs, list):
        if len(strs) == 1:
            return to_addr(strs[0])
        elif len(strs) == 2:
            host = strs[0]
            port = int(strs[1])
        else:
            raise ValueError
        return (host, port)
    elif isinstance(strs, str):
        strs = strs.split(':')
        if len(strs) == 1:
            host = strs[0]
            return (host, port)
        elif len(strs) == 2:
            return to_addr(strs)
        else:
            raise ValueError
    else:
        raise ValueError



import sh
from tempfile import mkstemp


def create_remote_file(text, ip, dst):
    _, temppath = mkstemp()
    with open(temppath, 'w') as f:
        f.write(text)

    sh.scp(temppath,
           f'root@{ip}:{dst}')



def partition(ls, size):
    """
    Returns a new list with elements
    of which is a list of certain size.

        >>> partition([1, 2, 3, 4], 3)
        [[1, 2, 3], [4]]
    """
    return [ls[i:i+size] for i in range(0, len(ls), size)]


def split(ls, sep=None):
    """
    Split a list by sep

        >>> split([1, None, None, 2, 3, None, 4])
        [[1], [2, 3], [4]]
    """
    res = []
    tmplist = []
    for elem in ls:
        if elem == sep:
            if tmplist:
                res.append(tmplist)
            tmplist = []
        else:
            tmplist.append(elem)
    else:
        res.append(tmplist)
    return res


if __name__ == '__main__':
    from math import ceil
    ls = [1, 2, 3, 4, 5]
    foo = partition(ls, 2)
    assert foo == [[1, 2], [3, 4], [5]]
    foo = partition(ls, ceil(len(ls)/3))
    assert foo == [[1, 2], [3, 4], [5]]
    ls = split([1, None, None, 2, 3, None, 4])
    assert ls == [[1], [2, 3], [4]]



#
# 帮助更好的编写命令行工具
#
import os
import sys
from functools import wraps
from termcolor import colored


class Error(Exception):
    pass


class EnvironError(Error):
    def __init__(self, variable, message=None):
        self.variable = variable
        if message is None:
            message = f'Error: 环境变量 `{variable}` 未设置'


def error(text):
    print(
        colored(text, 'red', attrs=['bold']),
        file=sys.stdout)


def warn(text):
    print(
        colored(text, 'red'),
        file=sys.stdout)


def environ_required(*environ_args, **environ_kargs):
    '''  examples:

        [root@localhost]# export A=3
        [root@localhost]# export B=4

        @environ_required('A', b='B')
        def main():
            print(A, b)

        # [out] 3 4

    '''
    def decorator(f):
        @wraps(f)
        def decorated_function(*args, **kwargs):
            environ_kargs.update({i: i for i in environ_args})
            for arg, environ_name in environ_kargs.items():
                try:
                    f.__globals__[arg] = os.environ[environ_name]
                except KeyError as e:
                    raise EnvironError(environ_name)
            return f(*args, **kwargs)
        return decorated_function
    return decorator



class FileWrapper:
    """Wrapper to convert file-like objects to iterables"""

    def __init__(self, filelike, blksize=8192):
        self.filelike = filelike
        self.blksize = blksize
        if hasattr(filelike, 'close'):
            self.close = filelike.close

    def __getitem__(self, key):
        data = self.filelike.read(self.blksize)
        if data:
            return data
        raise IndexError

    def __iter__(self):
        return self

    def __next__(self):
        data = self.filelike.read(self.blksize)
        if data:
            return data
        raise StopIteration
